****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest4.sas  
*
* Created (mm/dd/yyyy): 08/24/2015
* Last modified: 10/20/2018
* Version: 4.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create characteristics and medication of interest tables for pregnancy analyses.                                        
*   
*  Program inputs:  
*	- a dataset containing MOI drug codes 
*	- a dataset containing pregnancy codes
*	- a dataset containing defined group pregnancy and control episodes
*	- a dataset containing MOI claims
*             	                        
*  Program outputs:                                                                                                                                       
*	- Characteristics and Medication of Interest tables
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1	08/10/2016   AP (SOC)	Added underscore prefix to proc transpose (QCI-179)
*
*    2.0    02/16/2017   DM 		New design for QRP 4.0
*
*    2.1	05/05/2017	 AP			1. Added race and hispanic stratification for pregnancy cohort
*									2. Moved macro call into group loop
*
*    2.2    06/30/2017   RR			Only count MOI episodes that occur during trimester (QRP-395) 
*
*    3.0	10/03/2017	 AP			1. Recoded squared tables for scalability and to mirror other Types
*									2. Add geographic stratifications
*									3. Change PreTerm/PostTerm algorithm (QRP-388)
*
*    3.1    12/20/2017   AP			1. Add overall npts/episodes to _elig table (QRP-458)
*									2. Relabel race/hispanic stratifications (QRP-474)
*
*    3.2    03/14/2018   RR		    Changed Group length to $40 (QRP-495)
*
*    3.3    04/17/2018   AP		    Added gestational week table, t4cohortdef2, REMOVEMOI, prepreg parameters 
*									and Adjusted/Raw code counts (QRP-492, QRP-577, QRP-578)
*
*    3.4    05/22/2018   AP		    Fixed 2 bugs (DEV-1653, DEV-1657)
*
*    4.0    05/15/2018   AP			Added group looping and output strata based on USERSTRATA input file
*									Control group output is now optional
*
*    4.1    10/20/2018   AP			Moved creation of individual MOI metrics to separate macro
*
***************************************************************************************************;

%macro ms_cidatablest4();

	%put =====> MACRO CALLED: ms_cidatablest4 v4.1;

	*Macro produces 5 tables:
		1. t4_cida_preg
		2. t4_cida_nopreg
		3. t4_cida_elig
		4. t4_cida_preg_gestwk
		5. t4_cida_nopreg_gestwk;

	data _t4levels_preg
		 _t4levels_nopreg
		 _t4levels_elig
		 _t4levels_preg_gestwk
		 _t4levels_nopreg_gestwk
		 _t4levels;
		set userstrata;

		*Only available stratification for gestwk tables is moiname*gestwk;
		if tableID in ('t4preggestwk', 't4nopreggestwk') then do;
			levelvars = 'moiname gestwk';
		end;

		if lowcase(tableID) in ("t4preg") then output _t4levels_preg;
		if lowcase(tableID) in ("t4nopreg") then output _t4levels_nopreg;
		if lowcase(tableID) in ("t4elig") then output _t4levels_elig;
		if lowcase(tableID) in ("t4preggestwk") then output _t4levels_preg_gestwk;
		if lowcase(tableID) in ("t4nopreggestwk") then output _t4levels_nopreg_gestwk;

		if lowcase(tableID) in ("t4preg", "t4nopreg", "t4preggestwk", "t4nopreggestwk") then output _t4levels;
	run;

	%isdata(dataset=_t4levels);
	%if %eval(&nobs.>0) %then %do;

	/** Macro to compute aggregate metrics **/
	%macro MedicationUse(grouping=, level=);

		proc means data=_mstr nway noprint missing;
			var UsePre AnyT1 AnyT2 AnyT3 AnyT OnlyT1 OnlyT2 OnlyT3 AllT 
				SumAdjCntPre SumRawCntPre SumAdjCntAnyT1 SumAdjCntAnyT2
			 	SumAdjCntAnyT3 SumAdjCntAnyT SumAdjCntOnlyT1 SumAdjCntOnlyT2
			 	SumAdjCntOnlyT3 SumRawCntAnyT1 SumRawCntAnyT2
			 	SumRawCntAnyT3 SumRawCntAnyT SumRawCntOnlyT1 SumRawCntOnlyT2
			 	SumRawCntOnlyT3;
	   		class PatId DelNum &grouping.; 			
			output out=_MOI_by_strat (drop=_:) 
								   max(DelEpisodeInd)=Episodes
								   max(Has3Trim)=Episodes_3Trim 
								   min(TrimOnlyWashMet)=
								   max(UsePre AnyT1 AnyT2 AnyT3 AnyT AllT OnlyT1 OnlyT2 OnlyT3)=									       
								   sum(UsePre)=sumUsePre sum(AnyT1)=sumAnyT1 sum(AnyT2)=sumAnyT2
								   sum(AnyT3)=sumAnyT3 sum(AnyT)=sumAnyT sum(AllT)=sumAllT
								   sum(OnlyT1)=sumOnlyT1 sum(OnlyT2)=sumOnlyT2 sum(OnlyT3)=sumOnlyT3
								   sum(SumAdjCntPre SumRawCntPre SumAdjCntAnyT1 SumAdjCntAnyT2
										SumAdjCntAnyT3 SumAdjCntAnyT SumAdjCntOnlyT1 SumAdjCntOnlyT2
										SumAdjCntOnlyT3 SumRawCntAnyT1 SumRawCntAnyT2 SumRawCntAnyT3
										SumRawCntAnyT SumRawCntOnlyT1 SumRawCntOnlyT2 SumRawCntOnlyT3)=;
										
		run;

	    data _MOI_by_strat;
			set _MOI_by_strat;

			* Trimesters Only criterion;			
			if TrimOnlyWashMet=0 or AnyT2 or AnyT3 then OnlyT1=0;
			if TrimOnlyWashMet=0 or AnyT1 or AnyT3 then OnlyT2=0;
			if TrimOnlyWashMet=0 or AnyT1 or AnyT2 then OnlyT3=0;
			    
			if AnyT1 and AnyT2 and AnyT3 then do;
				AllT=1;
				sumAllT=sumAnyT;
				sumRawCntAllT = SumRawCntAnyT1+SumRawCntAnyT2+SumRawCntAnyT3;
				sumAdjCntAllT = SumAdjCntAnyT1+SumAdjCntAnyT2+SumAdjCntAnyT3;
			end;

			if OnlyT1=0 then do;
				sumOnlyT1=0;
				SumAdjCntOnlyT1=0;
				SumRawCntOnlyT1=0;
			end;
			if OnlyT2=0 then do;
				sumOnlyT2=0;
				SumAdjCntOnlyT2=0;
				SumRawCntOnlyT2=0;
			end;
			if OnlyT3=0 then do;
				sumOnlyT3=0;
				SumAdjCntOnlyT3=0;
				SumRawCntOnlyT3=0;
			end;
				
			*Fill in missing values;
			if missing(SumAdjCntPre) then SumAdjCntPre = 0;
			if missing(SumRawCntPre) then SumRawCntPre = 0;			
			if missing(SumAdjCntAnyT1) then SumAdjCntAnyT1 = 0;
			if missing(SumAdjCntAnyT2) then SumAdjCntAnyT2 = 0;
			if missing(SumAdjCntAnyT3) then SumAdjCntAnyT3 = 0;
			if missing(SumAdjCntAnyT) then SumAdjCntAnyT = 0;
			if missing(SumAdjCntAllT) then SumAdjCntAllT = 0;
			if missing(SumRawCntAnyT1) then SumRawCntAnyT1 = 0;
			if missing(SumRawCntAnyT2) then SumRawCntAnyT2 = 0;
			if missing(SumRawCntAnyT3) then SumRawCntAnyT3 = 0;
			if missing(SumRawCntAnyT) then SumRawCntAnyT = 0;
			if missing(SumRawCntAllT) then SumRawCntAllT = 0;

			if missing(AnyT) then AnyT = 0;
			if missing(SumAnyT) then SumAnyT = 0;
		run;

		proc means data=_MOI_by_strat nway noprint missing;
		    var Episodes Episodes_3Trim UsePre AnyT1 AnyT2 AnyT3 AnyT OnlyT1 OnlyT2 OnlyT3 AllT
				sumUsepre sumAnyT1 sumAnyT2 sumAnyT3 sumAnyT sumAllT sumOnlyT1 sumOnlyT2 sumOnlyT3
				SumAdjCntPre SumRawCntPre SumAdjCntAnyT1 SumAdjCntAnyT2
				SumAdjCntAnyT3 SumAdjCntAnyT SumAdjCntOnlyT1 SumAdjCntOnlyT2
				SumAdjCntOnlyT3 SumAdjCntAllT SumRawCntAnyT1 SumRawCntAnyT2
				SumRawCntAnyT3 SumRawCntAnyT SumRawCntOnlyT1 SumRawCntOnlyT2
				SumRawCntOnlyT3 SumRawCntAllT;
		    class PatId &grouping.;		
		    output out=_MOI_by_strat (drop=_:)  sum=;
		run;

		data _MOI_by_strat;
			set _MOI_by_strat;
			%if %index(&grouping., moiname) > 0 %then %do;
			  	Npts = 0;
			  	if Episodes gt 0 then Npts = 1;
			%end;
			%else %do; 
			  	Npts = 1;
            %end;
			if missing(patid) then npts = 0;
		run;
				
		proc means data=_MOI_by_strat nway noprint missing;
		    var Npts Episodes Episodes_3Trim UsePre AnyT1 AnyT2 AnyT3 AnyT OnlyT1 OnlyT2 OnlyT3 AllT
				sumUsePre sumAnyT1 sumAnyT2 sumAnyT3 sumAnyT sumAllT sumOnlyT1 sumOnlyT2 sumOnlyT3 
				SumAdjCntPre SumRawCntPre SumAdjCntAnyT1 SumAdjCntAnyT2
				SumAdjCntAnyT3 SumAdjCntAnyT SumAdjCntOnlyT1 SumAdjCntOnlyT2
				SumAdjCntOnlyT3 SumAdjCntAllT SumRawCntAnyT1 SumRawCntAnyT2
				SumRawCntAnyT3 SumRawCntAnyT SumRawCntOnlyT1 SumRawCntOnlyT2
				SumRawCntOnlyT3 SumRawCntAllT;
		    class &grouping.;		
		    output out=_MOI_by_strat(drop=_:) sum=;
	    run;

		%if %eval(&s.=1) %then %do;
			data _NumCounts;
				set _MOI_by_strat;
				format level $3. group $40.;

				level = "&level.";
				group = "&ITGROUP.";

				%if %index(&grouping., moiname)>0 %then %do;
				where MOINAME ne "";
				%end;
			run;
		%end;
		%else %do;
			data _NumCounts;
		    	set  _NumCounts
		        	 _MOI_by_strat(in=a 
					%if %index(&grouping., moiname)>0 %then %do;
						where=(MOINAME ne "") %end; );
		        if a then do;	
					Level="&level.";
					group = "&ITGROUP.";
				end;
			run;
		%end;

		proc datasets nowarn noprint lib=work;
			delete _MOI_by_strat;
			quit;
			
	%mend MedicationUse;

	** Loop through groups **;

	%do Group=1 %to &Ngroup.;

		data _null_;
			set infolder.&COHORTFILE.;
			if _n_ = &Group. then do;
				call symputx('ITGROUP',strip(cohortgrp));            

				*Age stratification;
				if missing(agestrat) then do;
					call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
				end;
				else do;
					call symputx("agestrat",agestrat);
				end;
			end;
		run;
		data _null_;
		    set infolder.&TYPE4FILE.;
		    where group="&ITGROUP.";
			if missing(prepregdays) then call symputx('prepregdays', 0);
			else call symputx('prepregdays', abs(prepregdays));
		run;

		%macro CreateCohortTable(file=);
		
			/********************************/
			/* t4_preg and t4_nopreg tables */
			/********************************/
			%isdata(dataset=_t4levels_&file.);    
			%if %eval(&nobs.>0) %then %do;

				*Add covariates to _concepi_mstr file;
				%isdata(dataset=DPLocal.&RUNID._mstr_&file.);
				%if %eval(&nobs.>0) %then %do;
					proc sql noprint;
						create table _concepi_mstr_&file._tri as
						select x.*
							%do j=1 %to &numcovars.;%str(, y.covar&j.)%end;
		   					%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(, y.covar&k.)%end;
						from DPLocal.&RUNID._concepi_mstr_&file._tri(where=(group="&ITGROUP.")) as x
						left join DPLocal.&RUNID._mstr_&file.(where=(group="&ITGROUP.")) as y
						on x.patid = y.patid and x.delnum=y.delnum;
					quit;
				%end;
				%else %do;
					data _concepi_mstr_&file._tri;
						set DPLocal.&RUNID._concepi_mstr_&file._tri(where=(group="&ITGROUP.")) ;
							%do j=1 %to &numcovars.;%str(covar&j.=.;)%end;
		   					%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k.=.;)%end;
					run;
				%end;

				%isdata(dataset=_t4levels_&file.);    
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t4levels_&file.;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					*Square table;
					%ms_squaredtableshell(squarevars =&levelvars.,
										  analysisvars=UsePre AnyT1 AnyT2 AnyT3 Any OnlyT1 OnlyT2 OnlyT3 AllT DelEpisodeInd Has3Trim,
										  groups="&itgroup",
										  analysis=N);

					proc sort nodupkey data=_mastersquare;
						by _ALL_;
					run;

					/*Merge _ConcEpisodes with _Square to obtain squared table*/
					data _mstr;
					    set _concepi_mstr_&file._tri(in=b)
					        _mastersquare (in=a);
							if a then do;
								TrimOnlyWashMet = -1;
							end;
							if b then do;
								month=month(episodeenddt);
							end;
					run;

					/*Summarize data */
					%MedicationUse(grouping=&levelvars., level=&levelid.);

						proc datasets nowarn noprint lib=work;
							delete _mstr _mastersquare:;
						quit;
				%end;

				proc datasets nowarn noprint lib=work;
					delete _tmpconcepis _ConcDispensings:;
				quit;
				
				*List of covariates;
				proc sql noprint;
					select distinct levelvars into: levelvars separated by ' '
					from _t4levels_&file.;
				quit;
				%nonrep(invar = levelvars, outvar = levelvars1);
					*put covariates into macro var;
				%do c = 1 %to %sysfunc(countw(&levelvars1.));
					%if %index(%scan(&levelvars1., &c.), covar) > 0 %then %do;
						%let covarstrat = &covarstrat. %scan(&levelvars1., &c.);
					%end;
				%end;

				data _NumCounts;
					retain Group MOIName Level AgeGroup Year zip3 state hhs_reg cb_reg zip_uncertain race hispanic PrePostInd PregDurCode &covarstrat.;
					length race hispanic zip_uncertain $1. zip3 state hhs_reg cb_reg $7.;
					set _NumCounts;
						*initialize missing vars to prevent w a r n i n g s;
						if _n_ = 1 then do;
						dsid = open("_NumCounts");
							if varnum(dsid,"race") = 0 then race ='';
							if varnum(dsid,"hispanic") = 0 then hispanic ='';
							if varnum(dsid,"zip_uncertain") = 0 then zip_uncertain ='';
							if varnum(dsid,"zip3") = 0 then zip3 ='';
							if varnum(dsid,"state") = 0 then state ='';
							if varnum(dsid,"hhs_reg") = 0 then hhs_reg ='';
							if varnum(dsid,"cb_reg") = 0 then cb_reg ='';

							if varnum(dsid,"agegroup") = 0 then agegroup ='';
							if varnum(dsid,"agegroupnum") = 0 then agegroupnum =.;
							if varnum(dsid,"year") = 0 then year =.;
							if varnum(dsid,"month") = 0 then month =.;
							if varnum(dsid,"PrePostInd") = 0 then PrePostInd ='';
							if varnum(dsid,"PregDurCode") = 0 then PregDurCode ='';
							if varnum(dsid,"MOIName") = 0 then MOIName ='';
							%if %str(&covarstrat.) ne %str() %then %do;
							%do cov = 1 %to %sysfunc(countw(&covarstrat.));	
							if varnum(dsid,"%scan(&covarstrat., &cov.)") = 0 then %scan(&covarstrat., &cov.) ='';
							%end;
							%end;
							rc= close(dsid);
						end;
						drop rc dsid;
				run;

				*remove labels;
				proc datasets lib=work nolist noprint;
		       		modify _NumCounts;
		            attrib _all_ label=''; *Remove labels;
		        quit;

				* Create final output;
				proc sort data=_NumCounts;			  
					by group Level MOIName AgeGroupNum Year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic PrePostInd PregDurCode &covarstrat.;
				run;

				data _NumCounts;
					retain group Level MOIName AgeGroupNum agegroup Year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic PrePostInd PregDurCode &covarstrat.;
					set _NumCounts;
				run;

				/*save to MSOC*/
				%if %eval(&group.=1) %then %do;
		    		data MSOC.&RUNID._t4_cida_&file.;
			        	set _NumCounts;
			        run;
				%end;
				%else %do;
					 proc append base=MSOC.&RUNID._t4_cida_&file.
			                    data=_NumCounts force;
			        run;
				%end;

				proc datasets nowarn noprint lib=work;
					delete _numcounts;
				quit;

			%end; /*t4_preg/t4_nopreg*/


			/**************************/
			/* Gestational Week table */
			/**************************/

			%if %index(&tables., t4&file.gestwk) > 0 %then %do;

				data _null_;
					set _t4levels_&file._gestwk;
					if _n_ = 1 then do;
						call symputx('levelid', levelid);
					end;
				run;

				*Square Gestational week - Only Squaring Gestwk during evaluation period.
				 Episodes that being prior to period will still be output ;

				data _squaredgestwk;
					do i = -&prepregdays. to 301;
						if i <0 then gestwk = int((i/7)-1);
						else if i>=0 then gestwk = int((i/7)+1);

						if gestwk ne lag(gestwk) then output;
					end;
					drop i;
				run;

				proc sort nodupkey data=infolder.&COHORTCODES. out=_MPsSquared(keep=T4_INDEX group rename=T4_INDEX=MOIName);
					by group T4_INDEX;
					where T4_INDEX =:"MP" and strip(group) = "&itgroup";
				run;

				data _MPsSquared;
					set _MPsSquared;
					by group MoiName;
					output;
					if last.group then do;		
						MoiName="";
						output;
					end;
				run;	

				proc sql noprint undo_policy=none;
					create table _squaredgestwk as
					select x.*,
						   y.*,
						   x.gestwk as gestwkend
					from _squaredgestwk as x,
					_MPsSquared(where=(MOIName ne "")) as y;
				quit;

				data _gestwkepis;
					length MOIName $10 group $40;
					set _squaredgestwk
						dplocal.&RUNID._concepi_mstr_&file._tri(in=a where=(patid ne "" and group="&ITGROUP."));

						if a and MOINAME ne "" then MOI = 1;
						else MOI = 0;
				run;

				*Create 1 row per gestational week to calculate overlap;
				data _gestwkepis_overlap;
					set _gestwkepis;
						if gestwk ne . then do;
							do i = gestwk to gestwkend;
								gestwk_overlap = i;
								output;
								end;
								drop i;
						end;
						else do;
							output;
						end;
				run;

				*Calculate MOI stats;
				%macro moi_by_gestwk_stats(datain=, var=, dataout=);
					proc means data=&datain. nway noprint missing;
					    var MOI AdjustedDisp RawDisp;
					    class PatId DelNum gestwk&var. MOIName ; 			
					    output out=&dataout. (drop=_:) sum(MOI)=MOIEpisodes&var.
														    sum(AdjustedDisp)=AdjCodeCount&var.
												   		 	sum(RawDisp)= RawCodeCount&var.;
				    run;

					proc means data=&dataout. nway noprint missing;
					    var MOIEpisodes&var. AdjCodeCount&var. RawCodeCount&var.;
					    class PatId gestwk&var. MOIName;		
					    output out=&dataout. (drop=_:)  sum=;
				    run;

					 data &dataout.;
					 	  set &dataout.;
						  MOIPatients&var. = 0;
						  if MOIEpisodes&var. gt 0 then MOIPatients&var. = 1;
						  if missing(patid) then MOIPatients&var. = 0;
					 run;
					
				    proc means data=&dataout. nway noprint missing;
				    var MOIEpisodes&var. MOIPatients&var. RawCodeCount&var. AdjCodeCount&var.;
				    class gestwk&var. MOIName;		
				    output out=&dataout. (drop=_:) sum=;
				    run;

					data &dataout.;
						set &dataout.;
						level = "610";
						where MOINAME ne "";
						if missing(RawCodeCount&var.) then RawCodeCount&var. = 0;
						if missing(AdjCodeCount&var.) then AdjCodeCount&var. = 0;
						if missing(MOIEpisodes&var.) then MOIEpisodes&var. = 0;
						if missing(MOIPatients&var.) then MOIPatients&var. = 0;
					run;
				%mend;
				%moi_by_gestwk_stats(datain =_gestwkepis, var=, dataout=_MOI_by_gestwk);
				%moi_by_gestwk_stats(datain =_gestwkepis_overlap, var=_overlap, dataout=_MOI_by_gestwk_overlap);

				*Overlap dispensings;
				data _gestwkdisp;
					length MOIName $10 group $40;
					set _squaredgestwk
						t4_dispensings_mstr_&file.(in=a where=(patid ne "" and group="&ITGROUP."));

						if a and MOINAME ne "" then MOI = 1;
						else MOI = 0;
				run;

				*Create 1 row per gestational week to calculate overlap;
				data _gestwkdisp_overlap;
					set _gestwkdisp;
						if gestwk ne . then do;
							do i = gestwk to gestwkend;
								gestwk_overlap = i;
								output;
								end;
								drop i;
						end;
						else do;
							output;
						end;
				run;

				%moi_by_gestwk_stats(datain =_gestwkdisp_overlap, var=_overlap, dataout=_Disp_by_gestwk_overlap);

				*Calculate PregStats;
				data _gestageepis;
					set dplocal.&runid._mstr_&file.(where=(group="&itgroup."));
					episode = 1;
					rename ga_birth = gestwk;
					keep episode patid ga_birth;
				run;

				data _gestageepis;
					set _gestageepis
						_squaredgestwk;
				run;

				proc means data=_gestageepis nway noprint missing;
					var episode;
					class PatId gestwk;
					output out=_gestageepis_by_wk1(drop=_:) max(episode) = PatWeek
															sum(episode) = EpiWeek;
				run;

				*To calculate PregEpisodes, each episode terminates during the gestational week.
				*To calculate PrePatients, each patient terminates in the gestational week of their longest episode;
				proc means data=_gestageepis_by_wk1 nway noprint missing;
					var gestwk PatWeek;
					class patid;
					output out=_gestagepat(drop=_:) max=;
				run;

				data _gestageepis_by_wk;
					set _gestageepis_by_wk1(drop=patweek)
						_gestagepat;
						if missing(patweek) then patweek = 0;
						if missing(EpiWeek) then EpiWeek = 0;
				run;

				proc means data=_gestageepis_by_wk nway noprint missing;
					var EpiWeek PatWeek;
					class gestwk;
					output out=_gestageepis_by_wk(drop=_:) sum=;
				run;

				proc means data=_gestageepis_by_wk nway noprint missing;
					var EpiWeek PatWeek;
					output out=_gestageepis_totals(drop=_:) sum(EpiWeek) = EpiWeekTotal
															sum(PatWeek) = PatWeekTotal;
				run;

				proc sql noprint;
					create table _pregepis_pat as
					select x.*,
						   y.*
					from _gestageepis_by_wk as x,
						 _gestageepis_totals as y
					order by gestwk;
				quit;

				data _pregepis_pat(keep=gestwk pregpatients pregepisodes);
					set _pregepis_pat;
		     		by gestwk;

					lagpat = lag(patweek);
					lagepi = lag(epiweek);

					if _n_ =1 then do;
						PregPatients = PatWeekTotal;
						PregEpisodes = EpiWeekTotal;
					end;
					else do;
						PregPatients = PregPatients - lagpat;
						PregEpisodes = PregEpisodes - lagepi;
					end;
				    retain PregEpisodes PregPatients;
				run;

				*Merge with _MOI_by_gestwk to assign to each MOI;
				proc sql noprint;
					create table _&file._gestwk as
					select "&itgroup" as group format=$40. length=40,
						   "&levelid." as level format=$3. length=3,
						   moi.MOIName label = "",
						   moi.gestwk,
						   moi.MOIEpisodes,
						   moi.MOIPatients,
						   moi.AdjCodeCount label = "",
						   moi.RawCodeCount label = "",
						   ovr.MOIEpisodes_overlap,
						   ovr.MOIPatients_overlap,
						   disp.AdjCodeCount_overlap label = "",
						   disp.RawCodeCount_overlap label = "",
						   preg.pregpatients,
						   preg.pregepisodes
					from _moi_by_gestwk as moi
					left join _pregepis_pat as preg on moi.gestwk = preg.gestwk 
					left join _MOI_by_gestwk_overlap as ovr on moi.gestwk=ovr.gestwk_overlap and moi.moiname = ovr.moiname
					left join _Disp_by_gestwk_overlap as disp on moi.gestwk=disp.gestwk_overlap and moi.moiname = disp.moiname 
					order by MOIName, gestwk;
				quit;

				/*save to MSOC*/
				%if %eval(&group.=1) %then %do;
				    data MSOC.&RUNID._T4_cida_&file._gestwk;
						set _&file._gestwk;
					run;
				%end;
				%else %do;
					proc append base=MSOC.&RUNID._T4_cida_&file._gestwk
					     data=_&file._gestwk force;
					run;
				%end;

			%end; *output gestational week table;

		%mend CreateCohortTable;

		%CreateCohortTable(file=preg);
		%if "&t4_nopreggroup." = "Y" %then %do;
		%CreateCohortTable(file=nopreg);
		%end;

		%end; /*loop through groups*/
	%end; /*output t4 tables*/

	*************
	* Eligtable *
	************;

	%isdata(dataset=_t4levels_elig);
	%if %eval(&nobs.>0) %then %do;

		*Traditional strata (eligdays) is continuous so will not be squared. However, adding ability to output non-squared custom strata;

		proc sql noprint;
			select distinct levelvars into: all_vars_to_square separated by ' '
			from _t4levels_elig;
		quit;

		%nonrep(invar = all_vars_to_square, outvar = vars_to_square);

		%macro CreateEligTable(grouping=, level=);
			proc means data=_alldel nway noprint missing;	 
				var episode;
			    class PatId &grouping. ;		
			    output out=_elig_by_strat (drop=_:) sum=;
		    run;

		    proc means data=_elig_by_strat nway noprint missing;
			    var Episode;
			    class &grouping.;		
			    output out=_Elig_by_strat (drop=_:) sum=episodes;
		    run;

			%if %eval(&s.=1) %then %do;
				data _eligcounts;
					set _Elig_by_strat;
						format level $3. group $40.;
						level = "&level.";
						group = "&ITGROUP.";
				run;
			%end;
			%else %do;
				data _eligcounts;
			    	set  _eligcounts
			        	 _Elig_by_strat(in=a);
			        if a then do;	
						Level="&level.";
						group = "&ITGROUP.";
					end;
				run;
			%end;

			proc datasets nowarn noprint lib=work;
				delete _Elig_by_strat;
			quit;

		%mend CreateEligTable;

		/*loop through groups and calculate eligible table*/

		%do Group=1 %to &Ngroup.;

			data _null_;
				set infolder.&COHORTFILE.;
				if _n_ = &Group. then do;
					call symputx('ITGROUP',strip(cohortgrp));            

					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;
				end;
			run;

			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t4levels_elig;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				%ms_squaredtableshell(squarevars =&levelvars.,
									  analysisvars=,
									  groups="&itgroup",
									  analysis=N);

				*Merge squared table with _alldeliveries;
				data _alldel;
					set dplocal.&runid._alldeliveries(in=a where=(group="&itgroup."))
					    _mastersquare(in=b);
					if a then do;
						episode=1;
					end;
				    else do;
				       episode=0;
				    end;
				run;
						
				/*Summarize data */
				%CreateEligTable(grouping= group &levelvars., level=&levelid.);

				proc datasets nowarn noprint lib=work;
					delete _alldel _mastersquare;
				quit;
			%end;

			*remove labels;
			proc datasets lib=work nolist noprint;
		    	modify _eligcounts;
		        attrib _all_ label=''; *Remove labels;
			quit;

			*List of all strata;
			proc sql noprint;
				select distinct levelvars into: eligstrat1 separated by ' '
				from _t4levels_elig;
			quit;
			%nonrep(invar = eligstrat1, outvar = eligstrat);

			* Create final output;
			proc sort data=_eligcounts;			  
				by group Level &eligstrat.;
			run;

			data _eligcounts;
				retain group Level &eligstrat.;
				set _eligcounts;
			run;

			/*save to MSOC*/
			%if %eval(&group.=1) %then %do;
		    	data MSOC.&RUNID._t4_cida_elig;
			       	set _eligcounts;
			    run;
			%end;
			%else %do;
				proc append base=MSOC.&RUNID._t4_cida_elig
			                data=_eligcounts force;
			    run;
			%end;

			proc datasets nowarn noprint lib=work;
				delete _eligcounts;
			quit;

		%end;  *group loop;
	%end;  *output elig table;

%put NOTE: ******** END OF MACRO: ms_cidatablest4 v4.0 ********;

%mend ms_cidatablest4;
