****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_concomitantuse.sas  
*
* Created (mm/dd/yyyy): 05/31/2018
* Last modified: 06/30/2019
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Evaluate concomitant use, identify events                                   
*   
*  Program inputs:  
*             	                        
*  Program outputs:    
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  06/30/19	   AP		Added CONCMINEPISDUR parameter and run time metrics
*
***************************************************************************************************;

%macro ms_concomitantuse();

	%put =====> MACRO CALLED: ms_concomitantuse v1.1; 

   	/*loop through number of analysisgrp*/	
	%isdata(dataset=infolder.&concfile.); 
	data _null_;
    call symputx("num_order","&nobs.");
    run;
	%put The number of analysis groups is &num_order.;

	%do r = 1 %to &num_order.;

		%ms_starttimer(timestart=concruntime);

		/*reset macro vars*/
		%let analysisgrp = ;
		%let primary = ;
		%let secondary = ;
		%let exporder = ;
		%let concfupwashper = ;
		%let concblackoutper = ;
		%let conccohortdef = ;
		%let concminepisdur = ;

		proc datasets nowarn noprint lib=work;
			delete _PtsMasterList _fup: _conc: _it: _GroupUtilFile;
		quit;

		/*1 Assign macro variables*/
			data _null_;
				set infolder.&concfile.;
				if _n_ = &r. then do;
					call symputx("analysisgrp", analysisgrp);
					call symputx("primary", lowcase(primary));
					call symputx("secondary", lowcase(secondary));
					call symputx("exporder", upcase(exporder));
					call symputx("concfupwashper", concfupwashper);
					if concblackoutper = . then concblackoutper = 0;
					call symputx("concblackoutper", concblackoutper);
					call symputx("conccohortdef", conccohortdef);
					call symputx("eventcount", conceventcount);
					call symputx("concminepisdur", concminepisdur);
				end;
			run;
			%put now looping on &analysisgrp: #&r.;
			%put primary episode: &primary.;
			%put secondary episode: &secondary.;

			*Integrity check - Age Groupings must be the same between primary and secondary groups;
			proc sql noprint;
				select count(distinct AgeStrat) into: cagestrat
				from COHORTFILE
				where lowcase(cohortgrp) in ("&primary", "&secondary");
			quit;
				
			%if %eval(&cagestrat > 1) %then %do;
				%put WARNING: Primary and Secondary Cohort Groups contain different age ranges. Age will be based on primary group;
			%end;


		/*2. Select Primary and Secondary Episodes*/
			data primary_episodes(drop=blackoutper);
				set dplocal.&runid._mstr;
				where lowcase(group) = "&primary.";
			run;

			*If more than 1 secondary GROUP, create secondary episodes by collapsing the individual group episodes into non-overlapping episodes.;
			%if %eval(%sysfunc(countw(&secondary.)) > 1) %then %do;
				data epis_to_collapse;
					set dplocal.&runid._mstr;
					where lowcase(group) in 
						(%do grpp = 1 %to %sysfunc(countw(&secondary.));
						%let currgrp = %scan(&secondary., &grpp.);
						%if &grpp. ne 1 %then %do; , %end; "&currgrp."
						%end;	
						);

					Expiredt = episodeenddt;
					keep patid indexdt expiredt;
				run;

				proc sort data=epis_to_collapse; 
					by patid indexdt expiredt; 
				run;

				*Collapse episodes;
				data epis_to_collapse;
					set epis_to_collapse;
					by patid;
					if first.PatId then do;
						LRunOutDate=Expiredt;
						gap=.;
						episode=1;
					end;
					else do;
						gap = indexdt - LRunOutDate - 1;
						if gap > 0 then do;
						episode=episode+1;
						LRunOutDate = Expiredt;
					end;
						else LRunOutDate = max(Expiredt,LRunOutDate);
					end;

					group = "&secondary.";
					format LRunOutDate date9.;
					retain episode LRunOutDate;
				run;

				proc means noprint data=epis_to_collapse nway missing;
					var indexdt ExpireDt;
					class group patid episode;
					output out=secondary_episodes(drop=_:) min(indexdt)=indexdt
														   max(ExpireDt)=EpisodeEndDt;
				run;
			%end;
			%else %do;
				data secondary_episodes(drop=blackoutper);
					set dplocal.&runid._mstr;
					where lowcase(group) = "&secondary.";
				run;
			%end;


		/*3 Create concomitant episodes, depending on value of ExpOrder, with at least 1 day at risk post blackout*/
			proc sql noprint;
				create table _PtsMasterList as
				select distinct prim.patid
							  , prim.indexdt as PrimIndexDt
							  , prim.EpisodeEndDt as PrimEpisodeEndDt
							  , sec.indexdt as SecIndexDt
							  , sec.EpisodeEndDt as SecEpisodeEndDt
							  , max(prim.indexdt, sec.indexdt) as ConcIndexDt format=date9.
							  , min(prim.EpisodeEndDt, sec.EpisodeEndDt) as ConcEpisodeEndDt format=date9.
							  , prim.enr_start
							  , prim.enr_end
							  , prim.birth_date
							  , prim.sex
							  , prim.race
							  , prim.hispanic
							  , prim.zip3
							  , prim.state
							  , prim.hhs_reg
							  , prim.cb_reg
							  , prim.IndexDtLookEndDt
							  , prim.IndexLook
							  ,	case when (missing(prim.zip_date) | max(prim.indexdt, sec.indexdt) < prim.zip_date) then 'Y'
									 else 'N'
								end as zip_uncertain
							  , &concblackoutper. as BlackoutPer
							  , &concfupwashper. as FupWashPer /*for pov56*/
							  , &concminepisdur. as concminepisdur
				from primary_episodes as prim
				inner join secondary_episodes as sec 
					on prim.patid = sec.patid
				where %MS_PeriodsOverlap(period1=PrimIndexDt PrimEpisodeEndDt, period2=SecIndexDt SecEpisodeEndDt) 
					and calculated ConcEpisodeEndDt - (calculated ConcIndexDt+calculated BlackoutPer) >=0
					%if "&exporder" = "P" %then %do;
					and PrimIndexDt < SecIndexDt
					%end;
					%if "&exporder" = "S" %then %do;
					and PrimIndexDt = SecIndexDt
					%end;
					;
				quit;

			*Calculate Age Group;
				data _null_;
					set COHORTFILE;
					where lowcase(cohortgrp) = "&primary.";
						*Age stratification;
						if missing(agestrat) then do;
							call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
						end;
						else do;
							call symputx("agestrat",agestrat);
						end;
				run;

  				%ms_agestrat(infile=_PtsMasterList,
		                 outfile=_PtsMasterList,
		                 startdt=birth_date,
		                 enddt=concindexdt,
		                 timestrat=&agestrat.);

			data _PtsMasterList
				 PtsList(keep=patid)
				 enr_conc(keep=patid enr_start enr_end);

				set _PtsMasterList(rename=concindexdt=indexdt rename=concepisodeenddt=episodeenddt);
				if strip(AgeGroup) ne "";
				Age = (indexdt-birth_date)/365.25;

				*Remove episodes that do not meet ConcFupWashPer criteria;
        		if Enr_Start <= IndexDt-max(0,FupWashPer) then MinEnrolMet=1;

				*Remove episodes that do not meet minimum episode duration;
				if EpisodeEndDt - IndexDt +1 >= concminepisdur then MinEpisDurMet = 1;

				if MinEnrolMet = 1 and MinEpisDurMet=1 ;
				drop prim: sec:;
			run;


		/*4 Process Event Codes */
		    data _diag 
		         _others;
			    set &COHORTCODES.;
			    where lowcase(group)=lowcase("&analysisgrp.") and conc_fup ne "NOT";
			    if CodeCat in("DX") then output _diag ;
			    else output _others;
		    run;

		    %ISDATA(dataset=_diag);
		    %IF %EVAL(&NOBS.>=1) %THEN %DO;
		        %MS_ProcessWildcards(InFile=_diag, 
		                             CodeVar=code, 
		                             OutFile=_diag);
		    %END;

		    *Create Loop-specific parameter files;
		    data _ITGroupParamNDC
		         _ITGroupParamMeds
		         _ITGroupParamLabs(rename=Group=RawGroup rename=CareSettingprincipal=RawCaresetting);
				set _diag
			    	_others;

				if missing(excludesupply) then excludesupply = 'N';

				*Output;
				if CodeCat="RX"          then output _ITGroupParamNDC;
				if CodeCat in("DX","PX") then output _ITGroupParamMeds;
				if CodeCat in("LB")      then output _ITGroupParamLabs;
		    run;

		    *Break out CaresettingPrincipal into Caresetting and Principal;
		    %ms_caresettingprincipal(InFile=_ITGroupParamMeds, 
		                             Var=CareSettingPrincipal, 
		                             OutFile=_ITGroupParamMeds);

		    *Extract records;
		    %ms_loopmeds(datafile=dplocal.meds,
		                 lookfile=_ITGroupParamMeds, 
						 multiplecodecat=Y,
		                 outfile=_temp_PreMeds);

		   	%ms_extractdrugs(datafile=dplocal.drugs,
		                     datavar=NDC ,
		                     lookfile=_ITGroupParamNDC, 
		                     lookvar=code , 
		                     outfile=_temp_PreDrugs);

			%ms_extractdrugs(datafile=dplocal.drugscomb,
		                     datavar=NDC ,
		                     lookfile=_ITGroupParamNDC, 
		                     lookvar=code , 
		                     outfile=_temp_PreDrugsComb);

			%ms_extractlabs(datafile=dplocal.labextract,
	        	            lookfile=_ITGroupParamLabs, 
	            	        outfile=_temp_prelabs);	

			*restricting to pts with concomitant episode;
			data _ITMeds(rename=DummyDate=ADate);
				if 0 then set PtsList;
				declare hash pt (hashexp:16, dataset:"PtsList");
				pt.definekey('PatId');
				pt.definedone();

				do until(eof1);
					set _temp_PreMeds end=eof1;
				    if pt.find()=0 then do;
						format DummyDate mmddyy10.;
						length DummyDate 4.;
						DummyDate=ADate;
						RxSup=1;
						RxAmt=1;
						drop ADate;
						output;
					end;
			    end;
			    stop;
			run;

		 	data _ITDrugs;
		        if 0 then set PtsList;
		        declare hash pt1 (hashexp:16, dataset:"PtsList");
		        pt1.definekey('PatId');
		        pt1.definedone();

		        do until(eof1);
		          set _temp_PreDrugs end=eof1;
		          if pt1.find()=0 then do;
				    if not missing(codesupply) then rxsup = codesupply;
					RxSupEndDt=Rxdate+RxSup-1; 	
					output;
				end;
		        end;
		        stop;
			run;

		 	data _ITDrugsComb;
		        if 0 then set PtsList;
		        declare hash pt1 (hashexp:16, dataset:"PtsList");
		        pt1.definekey('PatId');
		        pt1.definedone();

		        do until(eof1);
		          set _temp_PreDrugsComb end=eof1;
		          if pt1.find()=0 then do;
				    if not missing(codesupply) then rxsup = codesupply;
					RxSupEndDt=Rxdate+RxSup-1; 	
					output;
				end;
		        end;
		        stop;
			run;

			data _ITlabs(rename=DummyDate=ADate);
				if 0 then set PtsList;
				declare hash pt (hashexp:16, dataset:"PtsList");
				pt.definekey('PatId');
				pt.definedone();

				do until(eof1);
					set _temp_prelabs end=eof1;
				    if pt.find()=0 then do;
						format DummyDate mmddyy10.;
						length DummyDate 4.;
						DummyDate=ADate;
						RxSup=1;
						RxAmt=1;
						drop ADate;
						output;
					end;
			    end;
			    stop;
			run;
			
		    *Make sure all claims overlap enrollment span and only 
		     keep eligible exposures (overlaping Enr_Start Enr_End);
		  	%ms_shaveoutside(reffile=enr_conc,
		                     refstart=Enr_Start,
		                     refend=Enr_End,
		                     KeepPartBf=N,
		                     ToShaveFile=_ITDrugs,
		                     ToShaveStart=RxDate,
		                     ToShaveEnd=RxSupEndDt,
		                     outfile=_ITDrugs);

			%ISDATA(dataset=_ITDrugsComb);
			%if %eval(&NOBS.>=1) %then %do;
		    %ms_shaveoutside(reffile=enr_conc,
		                     refstart=Enr_Start,
		                     refend=Enr_End,
		                     KeepPartBf=N,
		                     ToShaveFile=_ITDrugsComb,
		                     ToShaveStart=RxDate,
		                     ToShaveEnd=RxSupEndDt,
		                     outfile=_ITDrugsComb(drop=Enr_Start Enr_end));
		    %end;
		    
		    %ms_shaveoutside(reffile=enr_conc,
		                     refstart=Enr_Start,
		                     refend=Enr_End,
		                     KeepPartBf=N,
		                     ToShaveFile=_ITMeds,
		                     ToShaveStart=ADate,
		                     ToShaveEnd=ADate,
		                     outfile=_ITMeds);
		    
		  	%ms_shaveoutside(reffile=enr_conc,
		                   refstart=Enr_Start,
		                   refend=Enr_End,
		                   KeepPartBf=N,
		                   ToShaveFile=_ITLabs,
		                   ToShaveStart=ADate,
		                   ToShaveEnd=ADate,
		                   outfile=_ITLabs);   

		   %ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
		    %IF %EVAL(&NOBS.>0) %THEN %DO;
		        data &STOCKPILINGFILE.;
		        set infolder.&STOCKPILINGFILE.;
		        where group="&analysisgrp.";
		        call symputx("SAMEDAY",strip(SameDay));
		        call symputx("SUPRANGE",strip(SupRange));
		        call symputx("AMTRANGE",strip(AmtRange));
		        call symputx("PERCENTDAYS",put(PercentDays,best.));
		        run;
		    %end;         

		    %MS_STOCKPILING(INFILE=_ITDrugs,	  		
		                    CLMDATE=RxDate,                            		
		                    CLMSUP=RxSup,                              		
		                    CLMAMT=RxAmt,	                           		
		        		    PROCFLAG=,                                 		
		        		    PERCENTDAYS=&PERCENTDAYS.,                          		
		        		    GROUPING=Group StockGroup T1_INDEX T2_INDEX T2_FUP T3_INDEX T3_FUP T4_INDEX T4_FUP T5_INDEX T6_INDEX CONC_FUP &InclExclVars.,                   		
		        		    SAMEDAY=&SAMEDAY.,                                 	
		        			SUPRANGE=&SUPRANGE.,									
		        			AMTRANGE=&AMTRANGE.,
		        		    ID=CODECAT &t6vars.,                                       		
		        		    OUTFILE=_ITDrugs,  		
		        			OUTFILEEXCL=_ITDrugsExcl
		                    );

		    data _ITDrugs;
		    set  _ITDrugs
		         _ITDrugsComb(in=a);
		    *ExpireDt does not exist in _ITDrugsComb;
		    if a then ExpireDt=Rxdate+RxSup-1; 	
		    run;

			*Need to reshave because Stockpiling may have pushed claims outside enrollment period;
		    %ISDATA(dataset=_ITDrugs);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;
		        %ms_shaveoutside(reffile=enr_conc,
		                       refstart=Enr_Start,
		                       refend=Enr_End,
		                       KeepPartBf=N,
		                       ToShaveFile=_ITDrugs,
		                       ToShaveStart=RxDate,
		                       ToShaveEnd=ExpireDt,
		                       outfile=_ITDrugs);
		    %END;	

	  		proc datasets library=work nowarn nolist;
		    	delete _diag _others _temp_: enr_conc;
		    quit;

			*Create _FUPEvent and _FUPWash datasets;
			data _FUPEvent(keep=PatId Adate ExpireDt Enr_Start codecat codetype code Excludesupply stockgroup enctype pdx)    
	      		 _FUPWash(keep=PatId Adate ExpireDt Enr_Start Excludesupply);                 							

			    set _ITDrugs(in=a rename=RxDate=ADate rename=ndc=code)
			        _ITMeds(in=b)
			        _ITLabs(in=c);

				if b or c then do;
					if not missing(codesupply) then RxSup = CodeSupply;
					else RxSup = 1;
				    Expiredt=Adate+RxSup-1;
				    RXAmt=1;
					NumDispensing = 1;
			    end;

	         	if CONC_FUP in('DEF') then output _FUPEvent;
	            if CONC_FUP in('IOC') then output _FUPWash;
			run;


		/*5 Identify Events */
			%ms_createpov56();


		/*6 Final Cohort */
	   		data _PtsMasterList;
				merge _PtsMasterList
					  _FirstEventDtInEpisode;
			    by PatId;
		    run;

			data _PtsMasterList;
				merge _PtsMasterList(in=a)
			          _EPisodesWithEvents(in=b);
				by Patid IndexDt;
				if a;
			run;

			data _PtsMasterList;
				set _PtsMasterList;
			    by Patid IndexDt;
		
		        tte=Min(EpisodeEndDt,Enr_End,FEventDt)-IndexDt -&concblackoutper. +1;

				%if "&ConcCohortDef" = "01" %then %do;
		            if first.PatId;
				%end;
				%if "&ConcCohortDef" = "03" %then %do;
		            if FEventDt_Any and IndexDt > FEventDt_Any then delete;
				%end;		        
 
				format AnalysisGrp $40.;
			    AnalysisGrp="&AnalysisGrp."; 
			    Year=Year(Indexdt);
				month=month(indexdt);

				EpisodeType = 'CONC';
				format LastLookFolloweddt date9.;
				LastLookFolloweddt = .;
			run;


		/*7. Calculate Utilization overlapping concomitant episode */
			data _GroupIndex;
				set gi&primary. 
					%do grpp = 1 %to %sysfunc(countw(&secondary.));
					gi%scan(&secondary., &grpp.)
					%end;
					;
			run;

			%ms_shaveoutside(reffile=_PtsMasterList,
		                     refstart=IndexDt,
		                     refend=EpisodeEndDt,
		                     KeepPartBf=Y,
		                     ToShaveFile=_GroupIndex,
		                     ToShaveStart=Adate,
		                     ToShaveEnd=ExpireDt,
		                     outfile=_GroupUtilFile);

		proc means data=_GroupUtilFile nway noprint;
			var RxSup RxAmt NumDispensing;
		    class PatId IndexDt;
		    output out=_GroupUtilFile(rename=_freq_=AdjustedDisp drop=_Type_) sum(RxSup RxAmt NumDispensing)=TotRxSup TotRxAmt RawDisp;
		run;	
	
    	data _PtsMasterList;
	    	merge _PtsMasterList (in=a)
	    	      _GroupUtilFile (in=b);
	    	by PatId IndexDt;
    		if a or b;*defensive;
		run;

		*Remove all labels;
	    proc datasets lib=work memtype=data nolist nowarn;
	    modify _PtsMasterList; 
	    attrib _all_ label=' '; 
	    run;

		/*8. Save to DPLOCAL */
		%if %eval(&r.=1) %then %do;
			data dplocal.&runid._mstr_concomitance;
				set _PtsMasterList;
			run;
		%end;
		%else %do;
			 data dplocal.&runid._mstr_concomitance;
	        	set dplocal.&runid._mstr_concomitance
	            	_PtsMasterList;
	        run;        
		%end;

		/*output runtime*/
		%ms_stoptimer(timestart=concruntime);
		%ms_outputruntimes(timevar=concruntime, step=%str(Concomitant Use Module), group=&analysisgrp., monitoringperiod=);

	%end; /*loop through analysis groups*/

	%put NOTE: ********END OF MACRO: ms_concomitantuse v1.1********;

%mend ms_concomitantuse;
