****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_moimetrics.sas  
*
* Created (mm/dd/yyyy): 10/20/2018
* Last modified: 06/30/2019
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create individual level medication of interest metrics for pregnancy analysis.                                        
*   
*  Program inputs:  
*	- a dataset containing MOI drug codes 
*	- a dataset containing pregnancy codes
*	- a dataset containing defined group pregnancy and control episodes
*	- a dataset containing MOI claims
*             	                        
*  Program outputs:                                                                                                                                       
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1	  06/30/19		AP		Added treescan variables to keep statement
*
***************************************************************************************************;

%macro ms_moimetrics(cohortfile=, outfile=);

	%put =====> MACRO CALLED: ms_moimetrics v1.1;

	*Create dataset with medical exposure during pregnancy;
	proc sql noprint;
		create table _ConcEpisodes as 
		select Prim.*, 
			   Sec.IndexDt as IndexDt2,
			   Sec.EpisodeEndDt as EpisodeEndDt2,
			   Sec.MOIName,
			   Sec.AdjustedDisp,
		   	   Sec.RawDisp
		from &cohortfile. as Prim
		left join _PtsMasterList2 as Sec
		on Prim.PatId=Sec.PatId 
		order by PatId;
	quit;

	* Do not count secondary cohort episodes (medical exposure)  
	  ending prior to "pre" period or beginning after the end of pregnancy period ;
	proc sql noprint;
		create table _PtsWithMOI as
		select distinct PatId
		from _ConcEpisodes
		where MOIName ne "" and %MS_PeriodsOverlap(period1 = indexDt-&prepregdays. episodeEndDt-&removeadate.,
	                                      			   period2=IndexDt2 episodeEndDt2)
		order by PatId;
	quit;

	data _ConcEpisodes_wMOI
		 _ConcEpisodes_woMOI;

		merge _ConcEpisodes
			  _PtsWithMOI(in=b);
		by PatId;		

		if IndexDt2 > EpisodeEndDt-&removeadate. | EpisodeEndDt2 < Indexdt-&prepregdays. then do;
			IndexDt2=.;
			EpisodeEndDt2=.;
			MOIName="";			
			AdjustedDisp=.;
			RawDisp=.;
		end;

		*Apply removal period (RemoveMOIStart/RemoveMOIEnd);
		%if "&removemoistart" ne . %then %do;
		if (EpisodeEndDt2 >= IndexDt+&removemoistart.) and (IndexDt2 <= IndexDt+&removemoiend.) then do;
			IndexDt2=.;
			EpisodeEndDt2=.;
			MOIName="";			
			AdjustedDisp=.;
			RawDisp=.;
		end;
		%end;

		if missing(MOINAME) then output _ConcEpisodes_woMOI;
		else output _ConcEpisodes_wMOI;
	run;

	%if "&t4cohortdef2" ne "02" %then %do;
		proc sort data = _ConcEpisodes_wMOI out=_ConcEpisodes_wMOI_1;
			by patid indexDt episodeEndDt DelNum MOIName IndexDt2 episodeEndDt2;
		run;

		/*T4cohortdef2 = 01: take the first exposure date during the pregnancy episode per MOI*/	
		/*T4cohortdef2 = 99: take the last exposure date during the pregnancy episode per MOI*/	
		data  _ConcEpisodes_wMOI;
	  		set _ConcEpisodes_wMOI_1;
	  		by patid indexDt episodeEndDt DelNum MOIName IndexDt2 episodeEndDt2;
			%if "&t4cohortdef2" = "01" %then %do;
			if first.MOIName;
			%end;
			%if "&t4cohortdef2" = "99" %then %do;
			if last.MOIName;
			%end;
		run;
	%end;

	data _ConcEpisodes;
		set _ConcEpisodes_wMOI _ConcEpisodes_woMOI;
	run;

	proc sort nodupkey data=_ConcEpisodes;
		by _ALL_;
	run;

	*Compute trimester use;
	data _ConcEpisodes;
		set _ConcEpisodes;
		UsePre=0;AnyT1=0;AnyT2=0;AnyT3=0;AnyT=0;OnlyT1=0;OnlyT2=0;OnlyT3=0;AllT=0;	
		TrimOnlyWashMet=0;	
		DelEpisodeInd=1;
					

		if strip(MOIName) ne "" then do;
			if 0<IndexDt-IndexDt2<=&prepregdays. or 0<IndexDt-EpisodeEndDt2<=&prepregdays. or 
			   (IndexDt2<IndexDt-&prepregdays. and EpisodeEndDt2>IndexDt) then UsePre=1;
			if IndexDt<=IndexDt2<=IndexDt+90 or IndexDt<=EpisodeEndDt2<=IndexDt+90 or 
			   (IndexDt2<IndexDt and EpisodeEndDt2>IndexDt+90) then AnyT1=1;
			if IndexDt+91<=IndexDt2<=IndexDt+180 or IndexDt+91<=EpisodeEndDt2<=IndexDt+180 or 
			   (IndexDt2<IndexDt+91 and EpisodeEndDt2>IndexDt+180) then AnyT2=1;
			if IndexDt+181<=IndexDt2<=EpisodeEndDt-&removeadate. or IndexDt+181<=EpisodeEndDt2<=EpisodeEndDt-&removeadate. or 
			   (IndexDt2<IndexDt+181 and EpisodeEndDt2>EpisodeEndDt-&removeadate.) then AnyT3=1;
			if AnyT1 or AnyT2 or AnyT3 then AnyT=1;
			if IndexDt<=EpisodeEndDt2<=IndexDt+90 then OnlyT1=1;
			if IndexDt+91<=IndexDt2 and EpisodeEndDt2<=IndexDt+180 then OnlyT2=1;
			if IndexDt+181<=IndexDt2<=EpisodeEndDt-&removeadate. then OnlyT3=1;
			if AnyT1 and AnyT2 and AnyT3 then AllT=1;	 							
			* Apply the washout criterion for the Trimesters Only statistics;
			* If ConcWashPer is missing, the exposure to the secondary cohort prior to the beginning 
		      of the pregnancy episode will be ignored when assessing the trimesters only statistics;
			TrimOnlyWashMet=1;
			if not missing(ConcWashPer) then do;
				if IndexDt > EpisodeEndDt2 and IndexDt - EpisodeEndDt2 -1 < ConcWashPer then TrimOnlyWashMet=0;
			end; 	

			*Gestational week of MOI;
			days1 = IndexDt2 - IndexDt;
			days2 = episodeenddt2 - IndexDt;
			if days1 <0 then gestwk =  int(((days1+1) / 7)-1);
			if days1 >=0 then gestwk =  int(((days1) / 7)+1);
			if days2 <0 then gestwkend =  int(((days2+1) / 7)-1);
			if days2 >=0 then gestwkend =  int(((days2) / 7)+1);
			drop days1 days2;
		end;
	run;	

	*Keep concurrent dispensings;
	proc sql noprint;
		create table _Concdispensings as
		select epi.patid,
			   epi.indexdt,
			   epi.delnum,
			   epi.episodeenddt,
			   epi.group,
			   epi.MOIName,
			   epi.IndexDt2,
			   epi.EpisodeEndDt2,
			   util.Adate,
			   util.expiredt,
			   util.numdispensing as RawDisp,
			   case when missing(epi.MOIName)=0 then 1
			   else 0
			   end as AdjustedDisp
		from _ConcEpisodes as epi
		left join _GroupUtilFile as util
		on epi.patid = util.patid and 
	   	   epi.MOIName = util.MoiName
		where %MS_PeriodsOverlap(period1 = epi.Indexdt2 epi.episodeEndDt2,
	                           	 period2 =util.adate util.expiredt);
	quit;

	data _ConcDispensings;
		set _Concdispensings;
		UsePreAdj=0;AnyT1Adj=0;AnyT2Adj=0;AnyT3Adj=0;AnyAdj=0;OnlyT1Adj=0;OnlyT2Adj=0;OnlyT3Adj=0;AllTAdj=0;	
		UsePreRaw=0;AnyT1Raw=0;AnyT2Raw=0;AnyT3Raw=0;AnyRaw=0;OnlyT1Raw=0;OnlyT2Raw=0;OnlyT3Raw=0;AllTRaw=0;	
					
		if 0<IndexDt-adate<=&prepregdays. or 0<IndexDt-expiredt<=&prepregdays. or 
		   (adate<IndexDt-&prepregdays. and expiredt>IndexDt) then do;
				UsePreAdj=AdjustedDisp;
				UsePreRaw=RawDisp;
		end;
		if IndexDt<=adate<=IndexDt+90 or IndexDt<=expiredt<=IndexDt+90 or 
		   (adate<IndexDt and expiredt>IndexDt+90) then do;
				AnyT1Adj=AdjustedDisp;
				AnyT1Raw=RawDisp;
		end;
		if IndexDt+91<=adate<=IndexDt+180 or IndexDt+91<=expiredt<=IndexDt+180 or 
		   (adate<IndexDt+91 and expiredt>IndexDt+180) then do;
				AnyT2Adj=AdjustedDisp;
				AnyT2Raw=RawDisp;
		end;
		if IndexDt+181<=adate<=EpisodeEndDt-&removeadate. or IndexDt+181<=expiredt<=EpisodeEndDt-&removeadate. or 
		   (adate<IndexDt+181 and expiredt>EpisodeEndDt-&removeadate.) then do;
				AnyT3Adj=AdjustedDisp;
				AnyT3Raw=RawDisp;
		end;

		if AnyT1Adj or AnyT2Adj or AnyT3Adj then AnyTAdj=1; else AnyTAdj = 0;
		if AnyT1Raw >= 1 or AnyT2Raw >= 1 or AnyT3Raw >= 1 then AnyTRaw=sum(AnyT1Raw, AnyT2Raw, AnyT3Raw); else AnyTRaw=0;

		if IndexDt<=expiredt<=IndexDt+90 then do;
			OnlyT1Adj=AdjustedDisp;
			OnlyT1Raw=RawDisp;
		end;
		if IndexDt+91<=adate and expiredt<=IndexDt+180 then do;
			OnlyT2Adj=AdjustedDisp;
			OnlyT2Raw=RawDisp;
		end;
		if IndexDt+181<=adate<=EpisodeEndDt-&removeadate. then do;
			OnlyT3Adj=AdjustedDisp;
			OnlyT3Raw=RawDisp;
		end;

		*Gestational week of code;
		days1 = Adate - IndexDt;
		days2 = expiredt - IndexDt;
		if days1 <0 then gestwk =  int(((days1+1) / 7)-1);
		if days1 >=0 then gestwk =  int(((days1) / 7)+1);
		if days2 <0 then gestwkend =  int(((days2+1) / 7)-1);
		if days2 >=0 then gestwkend =  int(((days2) / 7)+1);
		drop days1 days2;
	run;

	*Summarize and merge with _concepisodes;
	proc means data=_ConcDispensings nway noprint missing;
		var UsePreAdj AnyT1Adj AnyT2Adj AnyT3Adj AnyTAdj OnlyT1Adj OnlyT2Adj OnlyT3Adj AllTAdj 	
			UsePreRaw AnyT1Raw AnyT2Raw AnyT3Raw AnyTRaw OnlyT1Raw OnlyT2Raw OnlyT3Raw AllTRaw;
	   	class PatId indexdt delnum group moiname IndexDt2; 			
	   	output out=_ConcDispensings_summ (drop=_:) 
								   max(UsePreAdj AnyT1Adj AnyT2Adj AnyT3Adj AnyTAdj OnlyT1Adj OnlyT2Adj OnlyT3Adj 	
									   UsePreRaw AnyT1Raw AnyT2Raw AnyT3Raw AnyTRaw OnlyT1Raw OnlyT2Raw OnlyT3Raw)=		
									sum(UsePreAdj) = SumAdjCntPre
									sum(UsePreRaw) = SumRawCntPre
									sum(AnyT1Adj) = SumAdjCntAnyT1
									sum(AnyT2Adj) = SumAdjCntAnyT2
									sum(AnyT3Adj) = SumAdjCntAnyT3
									sum(AnyTAdj) = SumAdjCntAnyT
									sum(OnlyT1Adj) = SumAdjCntOnlyT1
									sum(OnlyT2Adj) = SumAdjCntOnlyT2
									sum(OnlyT3Adj) = SumAdjCntOnlyT3
 									sum(AnyT1Raw) = SumRawCntAnyT1
									sum(AnyT2Raw) = SumRawCntAnyT2
									sum(AnyT3Raw) = SumRawCntAnyT3
									sum(AnyTRaw) = SumRawCntAnyT
									sum(OnlyT1Raw) = SumRawCntOnlyT1
									sum(OnlyT2Raw) = SumRawCntOnlyT2
									sum(OnlyT3Raw) = SumRawCntOnlyT3;
	run;

	*Merge with _Concepisodes;
	proc sort data=_ConcDispensings_summ;
		by patid indexdt delnum group moiname indexdt2;
	run;
	proc sort data=_Concepisodes;
		by patid indexdt delnum group moiname indexdt2;
	run;

	data _Concepisodes&group.(keep=patid indexdt episodeenddt delnum group moiname indexdt2 pregdurcode has3trim agegroup year month
		     age agegroupnum episodeenddt2 prepostind ga_birth adjusteddisp rawdisp  enr_start enr_end IndexLook IndexDtLookEndDt IndexDtLookStart
			 sex race hispanic zip: state cb_reg hhs_reg use: any: only: sum: allt trimonlywashmet delepisodeind gestwk:
				%if "&outfile" = "mstr_preg" %then %do; csex linkedid cbirth_date cenr_start cenr_end birth_type matchmethod birth_date %end;);
		merge _ConcDispensings_summ _Concepisodes (in=b);
		by patid indexdt delnum group moiname indexdt2;
		if b;
	run;

	*Save to DPLOCAL;
	%if %eval(&group.=1) %then %do;
		data DPLocal.&RUNID._concepi_&outfile._tri;
			set _Concepisodes&group.;
	    run;
	%end;
	%else %do;
		proc append base=DPLocal.&RUNID._concepi_&outfile._tri
	                data=_Concepisodes&group. force;
	    run;
	%end;

	*Temporary dataset that contains 1 row per dispensing - for gestational week metrics;
	%if %eval(&group.=1) %then %do;
		data t4_dispensings_&outfile.;
			set _ConcDispensings;
	    run;
	%end;
	%else %do;
		proc append base=t4_dispensings_&outfile.
	                data=_ConcDispensings force;
	    run;
	%end;

	proc datasets nowarn noprint lib=work;
		delete _concepisodes: _concdispensings:;
	quit;

	%put NOTE: ******** END OF MACRO: ms_moimetrics v1.1 ********;

%mend ms_moimetrics;
