****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_setnumloopmacrovars.sas  
*
* Created (mm/dd/yyyy): 02/20/2017
* Last modified: 06/30/2019
* Version: 2.3
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This is a utility macro that is used to set macro variables related to a particular group during the numerators 
*	processing (ms_cidanum) based on the type of analysis being executed;                                     
*   
*  Program inputs:                                                                                   
*                                     
*  Program outputs:                                                                                                                                       
*                                                                                                  
*  PARAMETERS:    
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  05/04/2017	AP		1. Added race_out, hispanic_out, agestrat cohortfile parameters
*									2. Reordered assignment of &priordays_af/bf to fix QRP-375

*	1.2		  06/26/2017	AP		1. Added Type 5 analysis
*									2. Added covariate variables to Types 3 and 4 analysis
*									3. Added EpisodeGapType to Type 2 analysis
*
*	1.3		  09/14/2017	AP		Removed Condfrom/CondTo from &inclexclvars (QRP-437)
*
*	1.4		  09/22/2017	AP		Add Condfrom/CondTo from &inclexclvars (QRP-441)
*
*	1.5	      10/03/2017	AP	 	Added keep_blackout parameter (QRP-319) and moved GEOG to Cohort File (QRP-434)
*
*	1.6	      11/20/2017	AP	 	Removed keep_blackout parameter (QRP-466)
*
*	1.7	      12/21/2017	AP	 	1. Removed comorbidity window parameters (QRP-435)
*									2. Added CODEDAYS parameter (QRP-455)
*									3. Added censoring vars to Type1file (QRP-462)
*
*	1.8	      02/01/2018	AP	 	Removed variables related to post-index enrollment (QRP-496)
*
*   1.9       02/26/2018    RR		Initialize variables related to the unexposed group (QRP-495)
*
*   1.10      02/26/2018    AP		Removed NoFindGapDuplicate macro var (QRP-305)
*
*	2.0		  05/15/2018	AP	 	Moved ms_setnumloopmacrovars to separate file
*									Removed var related to specific stratifications (race, hispanic, geog) from Cohortfile					
*
*	2.1		  10/20/2018	AP	 	Added GraceDays and MIDAYSDIFF parameters to Type 4 File
*
*	2.2		  12/20/2018	AP	 	Added ExpExtPer to Type 5 file, and ExpExtPer2 to type 4 file
*
*	2.3		  06/30/2019	AP	 	Updated to reflect QRP 8.0.0 specifications
*
***************************************************************************************************;

%macro ms_setnumloopmacrovars();	

	%put =====> MACRO CALLED: ms_setnumloopmacrovars v2.3;

	data _null_;
		set cohortfile_for_enr;
		if _n_ = &Group. then do;
			call symputx('ITGROUP',strip(cohortgrp));            

			*Type of analysis;
			if upcase(type1)=:"Y" then call symputx('Type',1);
			if upcase(type2)=:"Y" then call symputx('Type',2);
			if upcase(type3)=:"Y" then call symputx('Type',3);
			if upcase(type4)=:"Y" then call symputx('Type',4);
			if upcase(type5)=:"Y" then call symputx('Type',5);
			if upcase(type6)=:"Y" then call symputx('Type',6);

			*Enrollment parameters;
				call symputx("enrollmentnum",put(enrollmentnum,best.));	

				call symputx("COVERAGE",strip(upcase(coverage)));

				*Minimum Enrollment days before exposure;
				call symputx("enrdays",put(enrdays,best.));							

				*Minimum Enrollment days after exposure index date;
				if EnrdaysAftInd ne . then do;
					call symputx("EnrdaysAftInd",put(EnrdaysAftInd,best.));
				end;
				else do;
					call symputx("EnrdaysAftInd",0);
				end;

				*Enrollment gap;
				if enrolgap ne . then call symputx("ENROLGAP",enrolgap);
				else call symputx("ENROLGAP","0"); 

				*Compound name for suffix of elig file;
				if missing(CHARTRES) then CHARTRES='N';
				call symputx("CHARTRES",CHARTRES);
				call symputx("RECONTYP",strip(upcase(coverage))||compress(CHARTRES,"_"));

			*Demographic criteria;
				*Age stratification;
				if missing(agestrat) then do;
				call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
				end;
				else do;
				call symputx("agestrat",agestrat);
				end;

				*Sex values applicable to demographics;
				if missing(Sex) then do;
				call symputx("DemogSex","'F' 'M' 'U' 'A'");
				end;
				else do;
				call symputx('DemogSex',upcase(Sex));
				end;

				*Race values applicable to demographics;
				if missing(Race) then do;
				call symputx("DemogRace","'0' '1' '2' '3' '4' '5'");
				end;
				else do;
				call symputx("DemogRace",upcase(Race));
				end;

				*Hispanic values applicable to demographics;
				if missing(Hispanic) then do;
				call symputx("DemogHispanic","'Y' 'N' 'U'");
				end;
				else do;
				call symputx("DemogHispanic",upcase(Hispanic));
				end;	
		end;
	run;

	%PUT &ITGROUP. &Type. &COVERAGE. &CHARTRES. &ENROLGAP. &RECONTYP. &enrdays. &enrdaysaftind. &agestrat. &demogsex. &demograce. &demoghispanic. &enrollmentnum.;   

	*HDPS variables;
	%ISDATA(dataset=HDPS);	
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		data _null_;
		set HDPS;
		where group="&ITGROUP";
	    if .<HDPSWinFROM <0 then call symputx("L1HDPSFROM",put(abs(HDPSWinFROM),best.));
		run;
		%put &L1HDPSFROM.;
	%END;

	****************************************;
	* Type 1 cohort definition     			;
	****************************************;
	%if %eval(&Type. = 1) %then %do;
	    data _CohortDef;
		    set infolder.&TYPE1FILE.;
		    where group="&ITGROUP.";
		      			
		    call symputx("COHORTDEF",T1CohortDef);
		    call symputx("WASHPER",T1WashPer);
			if upcase(censor_dth) eq "Y" then call symputx("censor_dth",1);
			else call symputx("censor_dth",0);
			call symputx("censdays_value_cat", censor_output_cat);

		    rename T1WashPer=WashPer;
	    run;		
	%end;

	****************************************;
	* Type 2 cohort definition     			;
	****************************************;
	%if %eval(&Type. = 2) %then %do;	
		data _CohortDef;
			set infolder.&TYPE2FILE.;
			where group="&ITGROUP.";

			if ittdays>0 then do;
				call symputx("ITT",1);
				*defensive coding;
				Episodegap=.;
				Episodegaptype = '';
				expextper=.;
				minepisdur=.;
				maxepisdur=.;
			end;
			else call symputx("ITT",0);

			if upcase(censor_dth) eq "Y" then call symputx("censor_dth",1);
			else call symputx("censor_dth",0);
			call symputx("censdays_value_cat", censor_output_cat);

			*never exposed;
			NEVEREXPOSEDCOHORT = strip(upcase(NEVEREXPOSEDCOHORT));
			call symputx("NEVEREXPOSEDCOHORT",upcase(NEVEREXPOSEDCOHORT));
			
			*defensive;
			if blackoutper = . then blackoutper = 0;
			if T2AtRiskStart =. then T2AtRiskStart = 0;

			call symputx("eventcount",eventcount);
			call symputx("COHORTDEF",T2CohortDef);
			call symputx("WASHPER",T2WashPer);
			call symputx("FUPWASHPER",T2FupWashPer);
			call symputx("ITTDAYS",ittdays);
			call symputx("ATRISKSTART", T2AtRiskStart);
			call symputx("BLACKOUTPER",BlackoutPer);
			call symputx("MINEPISDUR",MinEpisDur);
			call symputx("MAXEPISDUR",MaxEpisDur);
			call symputx("MINDAYSUPP",MinDaySupp);	
			rename T2WashPer=WashPer;
			rename T2FupWashPer=FupWashPer;

			EpisodeGapType = strip(upcase(EpisodeGapType));

			call symputx("MaxRiskTo", max(0, ITTDAYS, MAXEPISDUR));

			*Minimum Enrollment days after episode end;
			if EnrdaysAftEpi ne . then do;
				call symputx("EnrdaysAftEpi",put(EnrdaysAftEpi,best.));
			end;
			else do;
				call symputx("EnrdaysAftEpi",0);
			end;

			if ITTDAYS <=0 then do;
				if EpisodeGapType not in ('F', 'P') then do;
					EpisodeGapType = 'P';
					call symputx("EpisodeGapWarn",'Y');	
				end;
			end;
			
			drop group;
		run;		

		%if "&EpisodeGapWarn" = "Y" %then %do;
			%put WARNING: EPISODEGAPTYPE not set to F or P. Will default to P.;
		%end;

		%if %eval(&BLACKOUTPER.>0) & "&analysis"="tree" %then %do;
			%put WARNING: BLACKOUTPER should be left missing when ANALYSIS=TREE;
		%end;

	%end;

	****************************************;
	* Type 3 cohort definition     			;
	****************************************;
	%if %eval(&Type. = 3) %then %do;    
	    data _CohortDef;
		    set infolder.&TYPE3FILE.;
		    where group="&ITGROUP.";
		    
			if upcase(T3censor_dth) eq "Y" then call symputx("Censor_dth",1);
			else call symputx("Censor_dth",0);
		    call symputx("T3CtrlFrom",T3CtrlFrom);
		    call symputx("T3CtrlTo",T3CtrlTo);
		    call symputx("T3RiskFrom",T3RiskFrom);
		    call symputx("T3RiskTo",T3RiskTo);

			*Determine if no control window;
			if missing(t3ctrlfrom)| missing(t3ctrlto) then call symputx("T3CtrlWin", 'N');

			call symputx("MaxRiskTo", max(0, T3CtrlTo, T3RiskTo));

		    call symputx("ExclOnSameDay",T3ExclOnSameDay);        
		    call symputx("WASHPER",T3WashPer);
		    rename T3WashPer=WashPer;
	    run;

		%if "&T3CtrlWin" = "Y" %then %do;
			*Check if risk and control windows overlap;
			%if (&T3CtrlFrom. <= &T3RiskTo. & &T3CtrlTo. >= &T3RiskFrom.) | 
				(&T3RiskFrom. <= &T3CtrlTo. & &T3RiskTo. >= &T3CtrlFrom.) %then %do;			
				%put WARNING: The control window overlaps with the risk window.;
			%end;
		%end;
	%end;

	****************************************;
	* Type 4 cohort definition     			;
	****************************************;
	%if %eval(&Type. = 4) %then %do;
		data _CohortDef;
		    set infolder.&TYPE4FILE.;
		    where group="&ITGROUP.";

			expextper=0;	* needed for POV4 creation to avoid w.a.r.n.i.n.g.s.; 

			*Primary Cohort parameters;	
		    call symputx("COHORTDEF",T4CohortDef);
		    call symputx("WASHPER",T4WashPer);
		    call symputx("FUPWASHPER",T4FupWashper);
		    call symputx("ITTDAYS",ittdays);		
			call symputx("EnrDaysFloor",put(EnrDaysFloor,best.));		    

		    rename T4WashPer=WashPer;

			*Secondary Cohort parameters;
			if ittdays2>0 then do;
				call symputx("ITT2",1);
				EpisodeGapType2 = .;
				Episodegap2 = .;
				expextper2=.;
			end;
		    else call symputx("ITT2",0);
			
		    call symputx("WASHPER2",T4WashPer2);        
		    call symputx("ITTDAYS2",ittdays2);

			EpisodeGapType2 = upcase(EpisodeGapType2);

			if ITTDAYS2 <=0 then do;
			if EpisodeGapType2 not in ('F', 'P') then do;
				EpisodeGapType2 = 'P';
				call symputx("EpisodeGapWarn",'Y');	
			end;
			end;

			*MOI evaluation;
			call symputx('t4cohortdef2', t4cohortdef2);
			if ittdays2=1 then do;
				call symputx('removemoistart',removemoistart);
				call symputx('removemoiend',removemoiend);
			end;
			else do;
				call symputx('removemoistart',.);
				call symputx('removemoiend',.);
			end;

			if upcase(removemoiadate) = 'Y' then call symputx('removeadate', 1);
			else call symputx('removeadate', 0);
			
			if missing(prepregdays) then call symputx('prepregdays', 0);
			else call symputx('prepregdays', abs(prepregdays));

			*Linked Infant parameters;
			if gracedays = . then gracedays = 99999;
			if MIDAYSDIFF = . then MIDAYSDIFF = 99999;
		    call symputx("GRACEDAYS",gracedays);        
		    call symputx("MIDAYSDIFF",MIDAYSDIFF);        

		    rename T4WashPer2=WashPer2;
	    run;

		%if "&EpisodeGapWarn" = "Y" %then %do;
			%put WARNING: EPISODEGAPTYPE not set to F or P. Will default to P.;
		%end;

		*Verify that enrdays is sufficient to cover at least the whole episode duration;	
		%if %eval(&EnrDays. < &EnrDaysFloor.) %then %do;
			%put WARNING: The value of EnrDays is not high enough to ensure enrollment during episodes.A value of at least 294 days is required.;
		%end; 	

		%let preg = _preg;
	%end;

	****************************************;
	* Type 5 cohort definition     			;
	****************************************;
	%if %eval(&Type. = 5) %then %do;	
		data _CohortDef;
			set infolder.&TYPE5FILE.;
			where group="&ITGROUP.";

			if ittdays>0 then do;
				call symputx("ITT",1);
				call symputx("ITT2",1);
				*defensive coding;
				Episodegap=.;
				Episodegaptype = '';
			    expextper2=.;
			end;
			else do;
				call symputx("ITT",0);
				call symputx("ITT2",0);
			end;

			if upcase(censor_dth) eq "Y" then call symputx("censor_dth",1);
			else call symputx("censor_dth",0);

			call symputx("COHORTDEF",T5CohortDef);
			call symputx("WASHPER",T5WashPer);
			call symputx("ITTDAYS",ittdays);

			/*N/A to Type 5*/
			minepisdur=.;
			maxepisdur=.;
			mindaysupp=.;
			call symputx("MINEPISDUR",MinEpisDur);
			call symputx("MAXEPISDUR",MaxEpisDur);
			call symputx("MINDAYSUPP",MinDaySupp);

			blackoutper = .; /*defensive*/

			EpisodeGapType = upcase(EpisodeGapType);

			if ITTDAYS <=0 then do;
			if EpisodeGapType not in ('F', 'P') then do;
				EpisodeGapType = 'P';
				call symputx("EpisodeGapWarn",'Y');	
			end;
			end;

			rename T5WashPer=WashPer;

			/*For episode creation after index*/
			EpisodeGap2 = EpisodeGap;
			EpisodeGapType2 = EpisodeGapType;
			ITTdays2 = ITTDays;
			call symputx("ITTDAYS2",ittdays2);

			expextper2 = expextper;
			WashPer2 = 0;
			drop group;
		run;	

		%if "&EpisodeGapWarn" = "Y" %then %do;
			%put WARNING: EPISODEGAPTYPE not set to F or P. Will default to P.;
		%end;
	
	%end;

	****************************************;
	* Type 6 cohort definition     			;
	****************************************;

	%if %eval(&Type. = 6) %then %do;	
		data _CohortDef;
			set infolder.&TYPE6FILE.;
			where group="&ITGROUP.";

		    call symputx("ITT",0);
	        call symputx("ITT2",0);

			if upcase(censor_dth) eq "Y" then call symputx("censor_dth",1);
			else call symputx("censor_dth",0);

			call symputx("COHORTDEF",T6CohortDef);
	        call symputx("WASHPER",T6WashPer);
		    call symputx("WASHPER2",0);  
			call symputx("ITTDAYS",.);
			call symputx("ITTDAYS2",.);
	        call symputx("BLACKOUTPER",0);
	        call symputx("MINEPISDUR",MinEpisDur);
			call symputx("MAXEPISDUR",MaxEpisDur);
	        call symputx("MINDAYSUPP",MinDaySupp);
			call symputx("UPTAKEDATE",UPTAKEDATE);
			rename T6WashPer=WashPer;
			Washper2 = 0; /*Prevalent Cohort*/
			EpisodeGap2 = EpisodeGap; /*Prevalent Cohort*/
			expextper = 0;
			expextper2 = 0;
	        drop group;
	      
			EpisodeGapType = strip(upcase(EpisodeGapType));
			EpisodeGapType2 = EpisodeGapType;

			if EpisodeGapType not in ('F', 'P') then do;
				EpisodeGapType = 'P';
				call symputx("EpisodeGapWarn",'Y');	
			end;

			drop group;
		run;		

		%if "&EpisodeGapWarn" = "Y" %then %do;
			%put WARNING: EPISODEGAPTYPE not set to F or P. Will default to P.;
		%end;

		*Custom warning when T6CohortDef = 01;
		%if "&COHORTDEF." = "01" %then %do;
			%put WARNING: T6CohortDef is set to 01. Only first valid exposure episode for &ITGROUP. will be considered;
		%end;
		*Custom warning when Censor_dth = N;
		%if "&censor_dth." = "0" %then %do;
			%put WARNING: Censor_dth is set to N. Parameter should be set to Y for Type 6 Analysis;
		%end;

		*assign t6vars variable;
		%let t6vars=ndc productapprovaldate productmarketingdate otherproductdate;
	%end;

	************************************;
	* All Types - for Baseline Table 1	;
	************************************;
	*Get max covariatewindow from covariatecodes and MFU- even if looped at group level it will be the same since Covariate window same for both groups;
	%let MinCovFrom=.;
	%let MinMFUFrom=.;

	*Since input file is optional, make sure it exists;
	%IF %SYSFUNC(exist(infolder.&COVARIATECODES.))=1 %THEN %DO;
		*Select covariates codes (not combination of covariates);
		data _&COVARIATECODES.;
			set infolder.&COVARIATECODES.;
			where codecat not in ("CC");
		run;
			
		proc sql noprint;
			select max(-1*CovFrom) into: MinCovFrom
			from _&COVARIATECODES.(where=(. < CovFrom < 0));
		quit;

		*For secondary episode analysis: only force MinCovFrom enrollment for primary episode;
		%if %eval(&multievent.=1 | &overlap.=1) %then %do;
			data _tmp_primary;
				set secondaryinputfile(where=(upcase(primary)=upcase("&itgroup.")));
			run;
			%isdata(dataset=_tmp_primary);
			%if %eval(&nobs.=0) %then %do;
				%let MinCovFrom = 0;
			%end;

			proc datasets nowarn noprint lib=work;
				delete _tmp_primary;
			quit;
		%end;
	%END;
	%put &MinCovFrom.;

	%ISDATA(dataset=infolder.&mfufile.);	
	%if %eval(&nobs.>0) %then %do;
		proc sql noprint;
			select max(-1*MFUFrom) into: MinMFUFrom
			from infolder.&mfufile.(where=(. < MFUFROM < 0));
		quit;

	%END;
	%put &MinMFUFrom.;

	*Add w.a.r.n.i.n.g. if MinCovFrom or MinMFUFrom is bigger in absolute terms than other enrollment criteria;
	data _null_;
		MINFROM = max(0,&enrdays.,&washper.,&fupWashPer.); 
		MINCOVMFU = max(0, &MinCovFrom., &MinMFUFrom.);
	    call symputx("MINFROM",MINFROM);
	    call symputx("MINCOVMFU",MINCOVMFU);
	run;

	%IF %EVAL(&MINCOVMFU. > &MINFROM.) %THEN %DO;
		%put WARNING: many eligible patients may be lost due to forced enrollment criteria days used for covariates or MFU analysis;
	%END;
	%put &MINCOVMFU. &MINFROM.;


	*************************************************;
	* Secondary events analysis - Observation window ;
	*************************************************;

	%if %eval(&multievent.=1) | %eval(&overlap.=1) %then %do;

		proc sql noprint;
			select min(obsfrom) into: obs_from_index
			from secondaryinputfile
			where upcase(primary) = upcase("&itgroup.")
			and upcase(obsfromanchor) = "INDEX" and . < obsfrom <0;

			select min(obsfrom) into: obs_from_epiend
			from secondaryinputfile
			where upcase(primary) = upcase("&itgroup.")
			and upcase(obsfromanchor) = "EPISODEEND" and . < obsfrom <0;

			select max(obsto) into: obs_to_index
			from secondaryinputfile
			where upcase(primary) = upcase("&itgroup.")
			and upcase(obsfromanchor) = "INDEX" and obsto> 0;

			select max(obsto) into: obs_to_epiend
			from secondaryinputfile
			where upcase(primary) = upcase("&itgroup.")
			and upcase(obsfromanchor) = "EPISODEEND";
		quit;

	%end;

	****************************************;
	* TREE washout period		  			;
	****************************************;

	*take maximum treewashper for this group;
	%let maxtreewashper = 0;
	%if "&analysis" = "tree" %then %do;
		*if type =2, then determine analysisgrp from comparison file;
		%if %eval(&type.=2) %then %do;
			%isdata(dataset=infolder.&comparison.);
			%if %eval(&nobs.>0) %then %do;
				*Get analysisgrp;
				proc sql noprint;
					select quote(lowcase(strip(analysisgrp))) into: analysisgrplist separated by ','
					from infolder.&comparison.
					where lowcase(eoi)= lowcase("&itgroup.") | lowcase(ref)= lowcase("&itgroup.");
				quit;

				%if %str("&analysisgrplist") ne %str("") %then %do;
					proc sql noprint;
						select max(treewashper) into: maxtreewashper
						from infolder.&treefile.
						where lowcase(group) in (&analysisgrplist);
					quit;
				%end;
				%else %do;
					%put WARNING: COHORTGRP is not specified as EOI or REF in the comparison file;
				%end;
			%end;
			%else %do;
				%put ERROR: Comparison file is missing. This is a required input file when using tree extraction;
				%abort;
			%end;
		%end;

		*type 3;
		%if %eval(&type.=3) %then %do;
			proc sql noprint;
				select max(treewashper) into: maxtreewashper
				from infolder.&treefile.
				where lowcase(group) = lowcase("&itgroup.");
			quit;
		%end;

		%

		*type 4 - will evaluate in the macro %ms_createmicohorts;
	%end;

	%put NOTE: ******** END OF MACRO: ms_setnumloopmacrovars v2.2 ********;

%mend ms_setnumloopmacrovars;
